@@ -12,11 +12,12 @@
 // utilities and the self-extraction module.
 
 #if defined(WIN32)
+#define WIN32_LEAN_AND_MEAN
 #include <windows.h>
 #include <io.h>
 #include <conio.h>
 #include <shlobj.h>
-#elif defined(__GNUC__)
+#elif defined(__GNUC__) || defined(__sun)
 #include <glob.h>
 #include <unistd.h>
 #endif
@@ -139,7 +140,7 @@ char *filespec_ext (char *filespec)
 
     while (--cp >= filespec) {
 
-        if (*cp == '/' || *cp == ':')
+        if (*cp == '/')
             return NULL;
 
         if (*cp == '.') {
@@ -164,7 +165,7 @@ char *filespec_ext (char *filespec)
 // returned.                                                                //
 //////////////////////////////////////////////////////////////////////////////
 
-#if defined(__GNUC__) && !defined(WIN32)
+#if (defined(__GNUC__) || defined(__sun)) && !defined(WIN32)
 
 char *filespec_path (char *filespec)
 {
@@ -175,7 +176,7 @@ char *filespec_path (char *filespec)
     if (cp == filespec || filespec_wild (filespec))
         return NULL;
 
-    if (*--cp == '/' || *cp == ':')
+    if (*--cp == '/')
         return filespec;
 
     if (*cp == '.' && cp == filespec)
@@ -276,7 +277,7 @@ char *filespec_name (char *filespec)
     char *cp = filespec + strlen (filespec);
 
     while (--cp >= filespec)
-        if (*cp == '/' || *cp == ':')
+        if (*cp == '/')
             break;
 
     if (strlen (cp + 1))
@@ -297,13 +298,13 @@ char *filespec_name (char *filespec)
 
 static int is_second_byte (char *filespec, char *pos)
 {
-    uchar *cp = pos;
+    unsigned char *cp = pos;
 
     while (cp > filespec && ((cp [-1] >= 0x81 && cp [-1] <= 0x9f) ||
                              (cp [-1] >= 0xe0 && cp [-1] <= 0xfc)))
         cp--;
 
-    return (int)((uchar *)pos - cp) & 1;
+    return (int)((unsigned char *)pos - cp) & 1;
 }
 
 #endif
@@ -333,6 +334,10 @@ char yna (void)
             fprintf (stderr, "^C\n");
             exit (1);
         }
+        else if (key == EOF) {
+            fprintf (stderr, "\r\n");
+            exit (1);
+        }
         else if (key == '\r' || key == '\n') {
             if (choice) {
                 fprintf (stderr, "\r\n");
@@ -600,8 +605,24 @@ void finish_line (void)
 // Function to initialize console for intercepting ^C and ^Break.           //
 //////////////////////////////////////////////////////////////////////////////
 
+#include <signal.h>
+
+static int break_flag;
+
+static void int_handler(int s)
+{
+    break_flag = 1;
+}
+
 void setup_break (void)
 {
+    struct sigaction sigIntHandler;
+
+    break_flag = 0;
+    sigIntHandler.sa_handler = int_handler;
+    sigemptyset (&sigIntHandler.sa_mask);
+    sigIntHandler.sa_flags = 0;
+    sigaction (SIGINT, &sigIntHandler, NULL);
 }
 
 //////////////////////////////////////////////////////////////////////////////
@@ -610,7 +631,7 @@ void setup_break (void)
 
 int check_break (void)
 {
-    return 0;
+    return break_flag;
 }
 
 #endif
@@ -624,7 +645,7 @@ int DoReadFile (FILE *hFile, void *lpBuffer, uint32_t nNumberOfBytesToRead, uint
     *lpNumberOfBytesRead = 0;
 
     while (nNumberOfBytesToRead) {
-        bcount = (uint32_t) fread ((uchar *) lpBuffer + *lpNumberOfBytesRead, 1, nNumberOfBytesToRead, hFile);
+        bcount = (uint32_t) fread ((unsigned char *) lpBuffer + *lpNumberOfBytesRead, 1, nNumberOfBytesToRead, hFile);
 
         if (bcount) {
             *lpNumberOfBytesRead += bcount;
@@ -644,7 +665,7 @@ int DoWriteFile (FILE *hFile, void *lpBuffer, uint32_t nNumberOfBytesToWrite, ui
     *lpNumberOfBytesWritten = 0;
 
     while (nNumberOfBytesToWrite) {
-        bcount = (uint32_t) fwrite ((uchar *) lpBuffer + *lpNumberOfBytesWritten, 1, nNumberOfBytesToWrite, hFile);
+        bcount = (uint32_t) fwrite ((unsigned char *) lpBuffer + *lpNumberOfBytesWritten, 1, nNumberOfBytesToWrite, hFile);
 
         if (bcount) {
             *lpNumberOfBytesWritten += bcount;
@@ -661,12 +682,22 @@ int DoWriteFile (FILE *hFile, void *lpBuffer, uint32_t nNumberOfBytesToWrite, ui
 
 int64_t DoGetFileSize (FILE *hFile)
 {
-    struct stat64 statbuf;
+    LARGE_INTEGER Size;
+    HANDLE        fHandle;
 
-    if (!hFile || fstat64 (fileno (hFile), &statbuf) || !(statbuf.st_mode & S_IFREG))
+    if (hFile == NULL)
         return 0;
 
-    return statbuf.st_size;
+    fHandle = (HANDLE)_get_osfhandle(_fileno(hFile));
+    if (fHandle == INVALID_HANDLE_VALUE)
+        return 0;
+
+    Size.u.LowPart = GetFileSize(fHandle, &Size.u.HighPart);
+
+    if (Size.u.LowPart == INVALID_FILE_SIZE && GetLastError() != NO_ERROR)
+        return 0;
+
+    return (int64_t)Size.QuadPart;
 }
 
 #else
@@ -727,6 +758,27 @@ int DoTruncateFile (FILE *hFile)
 
 int DoDeleteFile (char *filename)
 {
-    return !remove (filename);
+    return filename ? !remove (filename) : 0;
 }
 
+/////////////////////////////////////////////////////////////////////////////////
+// Function to set the name of the console window. This is very handy for      //
+// displaying progress of batch operations with the console window minimized.  //
+/////////////////////////////////////////////////////////////////////////////////
+
+#ifdef WIN32
+
+void DoSetConsoleTitle (char *text)
+{
+    SetConsoleTitle (text);
+}
+
+#else
+
+void DoSetConsoleTitle (char *text)
+{
+    fprintf (stderr, "\033]0;%s\007", text);
+}
+
+#endif
+
