@@ -18,6 +18,7 @@
 #include <string.h>
 #include <math.h>
 
+//#define USE_OVERHEAD
 #define LOG_LIMIT 6912
 //#define EXTRA_DUMP
 
@@ -793,6 +794,18 @@ static void decorr_stereo_buffer (WavpackExtraInfo *info, int32_t *samples, int3
         decorr_stereo_pass_quick (samples, outsamples, num_samples, &dp, 1);
 }
 
+static int log2overhead (int first_term, int num_terms)
+{
+#ifdef USE_OVERHEAD
+    if (first_term > MAX_TERM)
+        return (8 + num_terms * 3) << 11;
+    else
+        return (4 + num_terms * 3) << 11;
+#else
+    return 0;
+#endif
+}
+
 static void recurse_stereo (WavpackContext *wpc, WavpackExtraInfo *info, int depth, int delta, uint32_t input_bits)
 {
     WavpackStream *wps = wpc->streams [wpc->current_stream];
@@ -826,6 +839,9 @@ static void recurse_stereo (WavpackContext *wpc, WavpackExtraInfo *info, int dep
         decorr_stereo_buffer (info, samples, outsamples, wps->wphdr.block_samples, depth);
         bits = log2buffer (outsamples, wps->wphdr.block_samples * 2, info->log_limit);
 
+        if (bits != (uint32_t) -1)
+            bits += log2overhead (info->dps [0].term, depth + 1);
+
         if (bits < info->best_bits) {
             info->best_bits = bits;
             CLEAR (wps->decorr_passes);
@@ -889,6 +905,9 @@ static void delta_stereo (WavpackContext *wpc, WavpackExtraInfo *info)
 
         bits = log2buffer (info->sampleptrs [i], wps->wphdr.block_samples * 2, info->log_limit);
 
+        if (bits != (uint32_t) -1)
+            bits += log2overhead (wps->decorr_passes [0].term, i);
+
         if (bits < info->best_bits) {
             lower = TRUE;
             info->best_bits = bits;
@@ -911,6 +930,9 @@ static void delta_stereo (WavpackContext *wpc, WavpackExtraInfo *info)
 
         bits = log2buffer (info->sampleptrs [i], wps->wphdr.block_samples * 2, info->log_limit);
 
+        if (bits != (uint32_t) -1)
+            bits += log2overhead (wps->decorr_passes [0].term, i);
+
         if (bits < info->best_bits) {
             info->best_bits = bits;
             CLEAR (wps->decorr_passes);
@@ -952,6 +974,9 @@ static void sort_stereo (WavpackContext *wpc, WavpackExtraInfo *info)
 
             bits = log2buffer (info->sampleptrs [i], wps->wphdr.block_samples * 2, info->log_limit);
 
+            if (bits != (uint32_t) -1)
+                bits += log2overhead (wps->decorr_passes [0].term, i);
+
             if (bits < info->best_bits) {
                 reversed = TRUE;
                 info->best_bits = bits;
@@ -1007,6 +1032,7 @@ static void analyze_stereo (WavpackContext *wpc, int32_t *samples, int do_sample
             decorr_stereo_pass_quick (info.sampleptrs [i], info.sampleptrs [i + 1], wps->wphdr.block_samples, info.dps + i, 1);
 
     info.best_bits = log2buffer (info.sampleptrs [info.nterms], wps->wphdr.block_samples * 2, 0) * 1;
+    info.best_bits += log2overhead (info.dps [0].term, i);
     memcpy (info.sampleptrs [info.nterms + 1], info.sampleptrs [i], wps->wphdr.block_samples * 8);
 
     if (wpc->config.extra_flags & EXTRA_BRANCHES)
@@ -1044,6 +1070,7 @@ static void analyze_stereo (WavpackContext *wpc, int32_t *samples, int do_sample
 static void stereo_add_noise (WavpackStream *wps, int32_t *lptr, int32_t *rptr)
 {
     int shaping_weight, new = wps->wphdr.flags & NEW_SHAPING;
+    short *shaping_array = wps->dc.shaping_array;
     int32_t error [2], temp, cnt;
 
     scan_word (wps, rptr, wps->wphdr.block_samples, -1);
@@ -1052,7 +1079,11 @@ static void stereo_add_noise (WavpackStream *wps, int32_t *lptr, int32_t *rptr)
 
     if (wps->wphdr.flags & HYBRID_SHAPE) {
         while (cnt--) {
-            shaping_weight = (wps->dc.shaping_acc [0] += wps->dc.shaping_delta [0]) >> 16;
+            if (shaping_array)
+                shaping_weight = *shaping_array++;
+            else
+                shaping_weight = (wps->dc.shaping_acc [0] += wps->dc.shaping_delta [0]) >> 16;
+
             temp = -apply_weight (shaping_weight, error [0]);
 
             if (new && shaping_weight < 0 && temp) {
@@ -1064,7 +1095,9 @@ static void stereo_add_noise (WavpackStream *wps, int32_t *lptr, int32_t *rptr)
             else
                 lptr [0] += (error [0] = nosend_word (wps, rptr [0], 0) - rptr [0]) + temp;
 
-            shaping_weight = (wps->dc.shaping_acc [1] += wps->dc.shaping_delta [1]) >> 16;
+            if (!shaping_array)
+                shaping_weight = (wps->dc.shaping_acc [1] += wps->dc.shaping_delta [1]) >> 16;
+
             temp = -apply_weight (shaping_weight, error [1]);
 
             if (new && shaping_weight < 0 && temp) {
@@ -1080,8 +1113,10 @@ static void stereo_add_noise (WavpackStream *wps, int32_t *lptr, int32_t *rptr)
             rptr += 2;
         }
 
-        wps->dc.shaping_acc [0] -= wps->dc.shaping_delta [0] * wps->wphdr.block_samples;
-        wps->dc.shaping_acc [1] -= wps->dc.shaping_delta [1] * wps->wphdr.block_samples;
+        if (!shaping_array) {
+            wps->dc.shaping_acc [0] -= wps->dc.shaping_delta [0] * wps->wphdr.block_samples;
+            wps->dc.shaping_acc [1] -= wps->dc.shaping_delta [1] * wps->wphdr.block_samples;
+        }
     }
     else
         while (cnt--) {
@@ -1163,7 +1198,7 @@ void execute_stereo (WavpackContext *wpc, int32_t *samples, int no_history, int
         wps->best_decorr = wps->mask_decorr = 0;
 
     for (pi = 0; pi < wps->num_passes;) {
-        WavpackDecorrSpec *wpds;
+        const WavpackDecorrSpec *wpds;
         int nterms, c, j;
 
         if (!pi)
@@ -1238,6 +1273,8 @@ void execute_stereo (WavpackContext *wpc, int32_t *samples, int no_history, int
                 break;
         }
 
+        size += log2overhead (wpds->terms [0], nterms);
+
         if (size < best_size) {
             memcpy (best_buffer, temp_buffer [j&1], buf_size);
             memcpy (wps->decorr_passes, save_decorr_passes, sizeof (struct decorr_pass) * MAX_NTERMS);
