@@ -19,7 +19,7 @@
 #define stricmp(x,y) _stricmp(x,y)
 #define fileno _fileno
 #else
-#define stricmp(x,y) strcasecmp(x,y)
+#define stricmp strcasecmp
 #endif
 
 #ifdef DEBUG_ALLOC
@@ -198,10 +198,9 @@ int WavpackDeleteTagItem (WavpackContext *wpc, const char *item)
         int i;
 
         for (i = 0; i < m_tag->ape_tag_hdr.item_count; ++i) {
-            int vsize, flags, isize;
+            int vsize, isize;
 
-            vsize = p[0] + (p[1] << 8) + (p[2] << 16) + (p[3] << 24); p += 4;
-            flags = p[0] + (p[1] << 8) + (p[2] << 16) + (p[3] << 24); p += 4;
+            vsize = p[0] + (p[1] << 8) + (p[2] << 16) + (p[3] << 24); p += 8;   // skip flags because we don't need them
             for (isize = 0; p[isize] && p + isize < q; ++isize);
 
             if (vsize < 0 || vsize > m_tag->ape_tag_hdr.length || p + isize + vsize + 1 > q)
@@ -230,13 +229,14 @@ int WavpackDeleteTagItem (WavpackContext *wpc, const char *item)
 // Once a APEv2 tag has been created with WavpackAppendTag(), this function is
 // used to write the completed tag to the end of the WavPack file. Note that
 // this function uses the same "blockout" function that is used to write
-// regular WavPack blocks, although that's where the similarity ends.
+// regular WavPack blocks, although that's where the similarity ends. It is also
+// used to write tags that have been edited on existing files.
 
 int WavpackWriteTag (WavpackContext *wpc)
 {
-    if (wpc->blockout)
+    if (wpc->blockout)      // this is the case for creating fresh WavPack files
         return write_tag_blockout (wpc);
-    else
+    else                    // otherwise we are editing existing tags (OPEN_EDIT_TAGS)
         return write_tag_reader (wpc);
 }
 
@@ -256,15 +256,25 @@ int load_tag (WavpackContext *wpc)
 
     CLEAR (*m_tag);
 
+    // This is a loop because we can try up to three times to look for an APEv2 tag. In order, we look:
+    //
+    //  1. At the end of the file for a APEv2 footer (this is the preferred location)
+    //  2. If there's instead an ID3v1 tag at the end of the file, try looking for an APEv2 footer right before that
+    //  3. If all else fails, look for an APEv2 header the the beginning of the file (use is strongly discouraged)
+
     while (1) {
 
-        // attempt to find an APEv2 tag either at end-of-file or before a ID3v1 tag we found
+        // seek based on specific location that we are looking for tag (see above list)
 
-        if (m_tag->id3_tag.tag_id [0] == 'T')
+        if (m_tag->tag_begins_file)                     // case #3
+            wpc->reader->set_pos_abs (wpc->wv_in, 0);
+        else if (m_tag->id3_tag.tag_id [0] == 'T')      // case #2
             wpc->reader->set_pos_rel (wpc->wv_in, -(int32_t)(sizeof (APE_Tag_Hdr) + sizeof (ID3_Tag)), SEEK_END);
-        else
+        else                                            // case #1
             wpc->reader->set_pos_rel (wpc->wv_in, -(int32_t)sizeof (APE_Tag_Hdr), SEEK_END);
 
+        // read a possible APEv2 tag header/footer and see if there's one there...
+
         if (wpc->reader->read_bytes (wpc->wv_in, &m_tag->ape_tag_hdr, sizeof (APE_Tag_Hdr)) == sizeof (APE_Tag_Hdr) &&
             !strncmp (m_tag->ape_tag_hdr.ID, "APETAGEX", 8)) {
 
@@ -278,37 +288,41 @@ int load_tag (WavpackContext *wpc)
                         ape_tag_items = m_tag->ape_tag_hdr.item_count;
                         ape_tag_length = m_tag->ape_tag_hdr.length;
 
-                        if (m_tag->id3_tag.tag_id [0] == 'T')
-                            m_tag->tag_file_pos = -(int32_t)sizeof (ID3_Tag);
-                        else
-                            m_tag->tag_file_pos = 0;
+                        // If this is a APEv2 footer (which is normal if we are searching at the end of the file)...
 
-                        m_tag->tag_file_pos -= ape_tag_length;
+                        if (!(m_tag->ape_tag_hdr.flags & APE_TAG_THIS_IS_HEADER)) {
 
-                        // if the footer claims there is a header present also, we will read that and use it
-                        // instead of the footer (after verifying it, of course) for enhanced robustness
+                            if (m_tag->id3_tag.tag_id [0] == 'T')
+                                m_tag->tag_file_pos = -(int32_t)sizeof (ID3_Tag);
+                            else
+                                m_tag->tag_file_pos = 0;
 
-                        if (m_tag->ape_tag_hdr.flags & APE_TAG_CONTAINS_HEADER)
-                            m_tag->tag_file_pos -= sizeof (APE_Tag_Hdr);
+                            m_tag->tag_file_pos -= ape_tag_length;
 
-                        wpc->reader->set_pos_rel (wpc->wv_in, m_tag->tag_file_pos, SEEK_END);
-                        memset (m_tag->ape_tag_data, 0, ape_tag_length);
+                            // if the footer claims there is a header present also, we will read that and use it
+                            // instead of the footer (after verifying it, of course) for enhanced robustness
 
-                        if (m_tag->ape_tag_hdr.flags & APE_TAG_CONTAINS_HEADER) {
-                            if (wpc->reader->read_bytes (wpc->wv_in, &m_tag->ape_tag_hdr, sizeof (APE_Tag_Hdr)) !=
-                                sizeof (APE_Tag_Hdr) || strncmp (m_tag->ape_tag_hdr.ID, "APETAGEX", 8)) {
-                                    free (m_tag->ape_tag_data);
-                                    CLEAR (*m_tag);
-                                    return FALSE;       // something's wrong...
-                            }
+                            if (m_tag->ape_tag_hdr.flags & APE_TAG_CONTAINS_HEADER)
+                                m_tag->tag_file_pos -= sizeof (APE_Tag_Hdr);
 
-                            little_endian_to_native (&m_tag->ape_tag_hdr, APE_Tag_Hdr_Format);
+                            wpc->reader->set_pos_rel (wpc->wv_in, m_tag->tag_file_pos, SEEK_END);
 
-                            if (m_tag->ape_tag_hdr.version != 2000 || m_tag->ape_tag_hdr.item_count != ape_tag_items ||
-                                m_tag->ape_tag_hdr.length != ape_tag_length) {
-                                    free (m_tag->ape_tag_data);
-                                    CLEAR (*m_tag);
-                                    return FALSE;       // something's wrong...
+                            if (m_tag->ape_tag_hdr.flags & APE_TAG_CONTAINS_HEADER) {
+                                if (wpc->reader->read_bytes (wpc->wv_in, &m_tag->ape_tag_hdr, sizeof (APE_Tag_Hdr)) !=
+                                    sizeof (APE_Tag_Hdr) || strncmp (m_tag->ape_tag_hdr.ID, "APETAGEX", 8)) {
+                                        free (m_tag->ape_tag_data);
+                                        CLEAR (*m_tag);
+                                        return FALSE;       // something's wrong...
+                                }
+
+                                little_endian_to_native (&m_tag->ape_tag_hdr, APE_Tag_Hdr_Format);
+
+                                if (m_tag->ape_tag_hdr.version != 2000 || m_tag->ape_tag_hdr.item_count != ape_tag_items ||
+                                    m_tag->ape_tag_hdr.length != ape_tag_length) {
+                                        free (m_tag->ape_tag_data);
+                                        CLEAR (*m_tag);
+                                        return FALSE;       // something's wrong...
+                                }
                             }
                         }
 
@@ -325,22 +339,37 @@ int load_tag (WavpackContext *wpc)
                 }
         }
 
+        // we come here if the search for the APEv2 tag failed (otherwise we would have returned with it)
+
         if (m_tag->id3_tag.tag_id [0] == 'T') {     // settle for the ID3v1 tag that we found
             CLEAR (m_tag->ape_tag_hdr);
             return TRUE;
         }
 
-        // look for ID3v1 tag if APEv2 tag not found during first pass
+        // if this was the search for the APEv2 tag at the beginning of the file (which is our
+        // last resort) then we have nothing, so return failure
+
+        if (m_tag->tag_begins_file) {
+            CLEAR (*m_tag);
+            return FALSE;
+        }
+
+        // If we get here, then we have failed the first APEv2 tag search (at end of file) and so now we
+        // look for an ID3v1 tag at the same position. If that succeeds, then we'll loop back and look for
+        // an APEv2 tag immediately before the ID3v1 tag, otherwise our last resort is to look for an
+        // APEv2 tag at the beginning of the file. These are strongly discouraged (and not editable) but
+        // they have been seen in the wild so we attempt to handle them here (at least well enough to
+        // allow a proper transcoding).
 
         m_tag->tag_file_pos = -(int32_t)sizeof (ID3_Tag);
         wpc->reader->set_pos_rel (wpc->wv_in, m_tag->tag_file_pos, SEEK_END);
 
         if (wpc->reader->read_bytes (wpc->wv_in, &m_tag->id3_tag, sizeof (ID3_Tag)) != sizeof (ID3_Tag) ||
             strncmp (m_tag->id3_tag.tag_id, "TAG", 3)) {
-                CLEAR (*m_tag);
-                return FALSE;       // neither type of tag found
-        }
-    }
+                m_tag->tag_begins_file = 1;     // failed ID3v1, so look for APEv2 at beginning of file
+                CLEAR (m_tag->id3_tag);
+            }
+    }  
 }
 
 // Return TRUE is a valid ID3v1 or APEv2 tag has been loaded.
@@ -355,6 +384,14 @@ int valid_tag (M_Tag *m_tag)
         return 0;
 }
 
+// Return FALSE if a valid APEv2 tag was only found at the beginning of the file (these are read-only
+// because they cannot be edited without possibly shifting the entire file)
+
+int editable_tag (M_Tag *m_tag)
+{
+    return !m_tag->tag_begins_file;
+}
+
 // Free the data for any APEv2 tag that was allocated.
 
 void free_tag (M_Tag *m_tag)
@@ -623,7 +660,24 @@ static int write_tag_reader (WavpackContext *wpc)
 {
     M_Tag *m_tag = &wpc->m_tag;
     int32_t tag_size = 0;
-    int result = TRUE;
+    int result;
+
+    // before we write an edited (or new) tag into an existing file, make sure it's safe and possible
+
+    if (m_tag->tag_begins_file) {
+        strcpy (wpc->error_message, "can't edit tags located at the beginning of files!");
+        return FALSE;
+    }
+
+    if (!wpc->reader->can_seek (wpc->wv_in)) {
+        strcpy (wpc->error_message, "can't edit tags on pipes or unseekable files!");
+        return FALSE;
+    }
+
+    if (!(wpc->open_flags & OPEN_EDIT_TAGS)) {
+        strcpy (wpc->error_message, "can't edit tags without OPEN_EDIT_TAGS flag!");
+        return FALSE;
+    }
 
     if (m_tag->ape_tag_hdr.ID [0] == 'A' && m_tag->ape_tag_hdr.item_count &&
         m_tag->ape_tag_hdr.length > sizeof (m_tag->ape_tag_hdr))
@@ -634,8 +688,7 @@ static int write_tag_reader (WavpackContext *wpc)
     if (m_tag->ape_tag_hdr.flags & APE_TAG_CONTAINS_HEADER)
         tag_size += sizeof (m_tag->ape_tag_hdr);
 
-    result = (wpc->open_flags & OPEN_EDIT_TAGS) && wpc->reader->can_seek (wpc->wv_in) &&
-        !wpc->reader->set_pos_rel (wpc->wv_in, m_tag->tag_file_pos, SEEK_END);
+    result = !wpc->reader->set_pos_rel (wpc->wv_in, m_tag->tag_file_pos, SEEK_END);
 
     if (result && tag_size < -m_tag->tag_file_pos) {
         int nullcnt = -m_tag->tag_file_pos - tag_size;
