@@ -3,6 +3,7 @@
 //                  Hybrid Lossless Wavefile Compressor                   //
 //              Copyright (c) 1998 - 2006 Conifer Software.               //
 //                          All Rights Reserved.                          //
+//      Distributed under the BSD Software License (see license.txt)      //
 ////////////////////////////////////////////////////////////////////////////
 
 // words.c
@@ -40,6 +41,8 @@ int32_t dump_alloc (void);
 
 //////////////////////////////// local macros /////////////////////////////////
 
+#define USE_NEXT8_OPTIMIZATION  // we normally want this, but code is easier to understand without it
+
 #define LIMIT_ONES 16   // maximum consecutive 1s sent for "div" data
 
 // these control the time constant "slow_level" which is used for hybrid mode
@@ -118,7 +121,7 @@ const char nbits_table [] = {
     8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8      // 240 - 255
 };
 
-static const uchar log2_table [] = {
+static const unsigned char log2_table [] = {
     0x00, 0x01, 0x03, 0x04, 0x06, 0x07, 0x09, 0x0a, 0x0b, 0x0d, 0x0e, 0x10, 0x11, 0x12, 0x14, 0x15,
     0x16, 0x18, 0x19, 0x1a, 0x1c, 0x1d, 0x1e, 0x20, 0x21, 0x22, 0x24, 0x25, 0x26, 0x28, 0x29, 0x2a,
     0x2c, 0x2d, 0x2e, 0x2f, 0x31, 0x32, 0x33, 0x34, 0x36, 0x37, 0x38, 0x39, 0x3b, 0x3c, 0x3d, 0x3e,
@@ -137,7 +140,7 @@ static const uchar log2_table [] = {
     0xf4, 0xf5, 0xf6, 0xf7, 0xf7, 0xf8, 0xf9, 0xf9, 0xfa, 0xfb, 0xfc, 0xfc, 0xfd, 0xfe, 0xff, 0xff
 };
 
-static const uchar exp2_table [] = {
+static const unsigned char exp2_table [] = {
     0x00, 0x01, 0x01, 0x02, 0x03, 0x03, 0x04, 0x05, 0x06, 0x06, 0x07, 0x08, 0x08, 0x09, 0x0a, 0x0b,
     0x0b, 0x0c, 0x0d, 0x0e, 0x0e, 0x0f, 0x10, 0x10, 0x11, 0x12, 0x13, 0x13, 0x14, 0x15, 0x16, 0x16,
     0x17, 0x18, 0x19, 0x19, 0x1a, 0x1b, 0x1c, 0x1d, 0x1d, 0x1e, 0x1f, 0x20, 0x20, 0x21, 0x22, 0x23,
@@ -156,6 +159,7 @@ static const uchar exp2_table [] = {
     0xea, 0xec, 0xed, 0xee, 0xf0, 0xf1, 0xf2, 0xf4, 0xf5, 0xf6, 0xf8, 0xf9, 0xfa, 0xfc, 0xfd, 0xff
 };
 
+#ifdef USE_NEXT8_OPTIMIZATION
 static const char ones_count_table [] = {
     0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,5,
     0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,6,
@@ -166,6 +170,7 @@ static const char ones_count_table [] = {
     0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,5,
     0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,8
 };
+#endif
 
 ///////////////////////////// executable code ////////////////////////////////
 
@@ -238,7 +243,7 @@ void word_set_bitrate (WavpackStream *wps)
 
 void write_entropy_vars (WavpackStream *wps, WavpackMetadata *wpmd)
 {
-    uchar *byteptr;
+    unsigned char *byteptr;
     int temp;
 
     byteptr = wpmd->data = malloc (12);
@@ -260,7 +265,7 @@ void write_entropy_vars (WavpackStream *wps, WavpackMetadata *wpmd)
         *byteptr++ = temp >> 8;
     }
 
-    wpmd->byte_length = (int32_t)(byteptr - (uchar *) wpmd->data);
+    wpmd->byte_length = (int32_t)(byteptr - (unsigned char *) wpmd->data);
     read_entropy_vars (wps, wpmd);
 }
 
@@ -273,7 +278,7 @@ void write_entropy_vars (WavpackStream *wps, WavpackMetadata *wpmd)
 
 void write_hybrid_profile (WavpackStream *wps, WavpackMetadata *wpmd)
 {
-    uchar *byteptr;
+    unsigned char *byteptr;
     int temp;
 
     word_set_bitrate (wps);
@@ -308,7 +313,7 @@ void write_hybrid_profile (WavpackStream *wps, WavpackMetadata *wpmd)
         }
     }
 
-    wpmd->byte_length = (int32_t)(byteptr - (uchar *) wpmd->data);
+    wpmd->byte_length = (int32_t)(byteptr - (unsigned char *) wpmd->data);
     read_hybrid_profile (wps, wpmd);
 }
 
@@ -320,7 +325,7 @@ void write_hybrid_profile (WavpackStream *wps, WavpackMetadata *wpmd)
 
 int read_entropy_vars (WavpackStream *wps, WavpackMetadata *wpmd)
 {
-    uchar *byteptr = wpmd->data;
+    unsigned char *byteptr = wpmd->data;
 
     if (wpmd->byte_length != ((wps->wphdr.flags & MONO_DATA) ? 6 : 12))
         return FALSE;
@@ -345,10 +350,13 @@ int read_entropy_vars (WavpackStream *wps, WavpackMetadata *wpmd)
 
 int read_hybrid_profile (WavpackStream *wps, WavpackMetadata *wpmd)
 {
-    uchar *byteptr = wpmd->data;
-    uchar *endptr = byteptr + wpmd->byte_length;
+    unsigned char *byteptr = wpmd->data;
+    unsigned char *endptr = byteptr + wpmd->byte_length;
 
     if (wps->wphdr.flags & HYBRID_BITRATE) {
+        if (byteptr + (wps->wphdr.flags & MONO_DATA ? 2 : 4) > endptr)
+            return FALSE;
+
         wps->w.c [0].slow_level = exp2s (byteptr [0] + (byteptr [1] << 8));
         byteptr += 2;
 
@@ -358,6 +366,9 @@ int read_hybrid_profile (WavpackStream *wps, WavpackMetadata *wpmd)
         }
     }
 
+    if (byteptr + (wps->wphdr.flags & MONO_DATA ? 2 : 4) > endptr)
+        return FALSE;
+
     wps->w.bitrate_acc [0] = (int32_t)(byteptr [0] + (byteptr [1] << 8)) << 16;
     byteptr += 2;
 
@@ -367,6 +378,9 @@ int read_hybrid_profile (WavpackStream *wps, WavpackMetadata *wpmd)
     }
 
     if (byteptr < endptr) {
+        if (byteptr + (wps->wphdr.flags & MONO_DATA ? 2 : 4) > endptr)
+            return FALSE;
+
         wps->w.bitrate_delta [0] = exp2s ((short)(byteptr [0] + (byteptr [1] << 8)));
         byteptr += 2;
 
@@ -473,9 +487,8 @@ int32_t FASTCALL send_word (WavpackStream *wps, int32_t value, int chan)
                 return 0;
             }
         }
-        else if (value) {
+        else if (value)
             putbit_0 (&wps->wvbits);
-        }
         else {
             c->slow_level -= (c->slow_level + SLO) >> SLS;
             CLEAR (wps->w.c [0].median);
@@ -584,9 +597,8 @@ int32_t FASTCALL send_word (WavpackStream *wps, int32_t value, int chan)
         uint32_t extras = bitset [bitcount] - maxcode - 1;
 
         if (bitcount) {
-            if (code < extras) {
+            if (code < extras)
                 putbits (code, bitcount - 1, &wps->wvcbits);
-            }
             else {
                 putbits ((code + extras) >> 1, bitcount - 1, &wps->wvcbits);
                 putbit ((code + extras) & 1, &wps->wvcbits);
@@ -632,9 +644,8 @@ void send_words_lossless (WavpackStream *wps, int32_t *buffer, int32_t nsamples)
                     continue;
                 }
             }
-            else if (value) {
+            else if (value)
                 putbit_0 (&wps->wvbits);
-            }
             else {
                 CLEAR (wps->w.c [0].median);
                 CLEAR (wps->w.c [1].median);
@@ -728,9 +739,8 @@ void flush_word (WavpackStream *wps)
     if (wps->w.zeros_acc) {
         int cbits = count_bits (wps->w.zeros_acc);
 
-        while (cbits--) {
+        while (cbits--)
             putbit_1 (&wps->wvbits);
-        }
 
         putbit_0 (&wps->wvbits);
 
@@ -751,9 +761,8 @@ void flush_word (WavpackStream *wps)
             wps->w.holding_one -= LIMIT_ONES;
             cbits = count_bits (wps->w.holding_one);
 
-            while (cbits--) {
+            while (cbits--)
                 putbit_1 (&wps->wvbits);
-            }
 
             putbit_0 (&wps->wvbits);
 
@@ -1016,6 +1025,7 @@ int32_t FASTCALL get_word (WavpackStream *wps, int chan, int32_t *correction)
     if (wps->w.holding_zero)
         ones_count = wps->w.holding_zero = 0;
     else {
+#ifdef USE_NEXT8_OPTIMIZATION
         if (wps->wvbits.bc < 8) {
             if (++(wps->wvbits.ptr) == wps->wvbits.end)
                 wps->wvbits.wrap (&wps->wvbits);
@@ -1061,6 +1071,34 @@ int32_t FASTCALL get_word (WavpackStream *wps, int chan, int32_t *correction)
             wps->wvbits.bc -= (ones_count = ones_count_table [next8]) + 1;
             wps->wvbits.sr >>= ones_count + 1;
         }
+#else
+        for (ones_count = 0; ones_count < (LIMIT_ONES + 1) && getbit (&wps->wvbits); ++ones_count);
+
+        if (ones_count >= LIMIT_ONES) {
+            uint32_t mask;
+            int cbits;
+
+            if (ones_count == (LIMIT_ONES + 1))
+                return WORD_EOF;
+
+            for (cbits = 0; cbits < 33 && getbit (&wps->wvbits); ++cbits);
+
+            if (cbits == 33)
+                return WORD_EOF;
+
+            if (cbits < 2)
+                ones_count = cbits;
+            else {
+                for (mask = 1, ones_count = 0; --cbits; mask <<= 1)
+                    if (getbit (&wps->wvbits))
+                        ones_count |= mask;
+
+                ones_count |= mask;
+            }
+
+            ones_count += LIMIT_ONES;
+        }
+#endif
 
         if (wps->w.holding_one) {
             wps->w.holding_one = ones_count & 1;
@@ -1106,6 +1144,8 @@ int32_t FASTCALL get_word (WavpackStream *wps, int chan, int32_t *correction)
         }
     }
 
+    low &= 0x7fffffff;
+    high &= 0x7fffffff;
     mid = (high + low + 1) >> 1;
 
     if (!c->error_limit)
@@ -1190,6 +1230,7 @@ int32_t get_words_lossless (WavpackStream *wps, int32_t *buffer, int32_t nsample
         if (wps->w.holding_zero)
             ones_count = wps->w.holding_zero = 0;
         else {
+#ifdef USE_NEXT8_OPTIMIZATION
             int next8;
 
             if (bs->bc < 8) {
@@ -1237,7 +1278,34 @@ int32_t get_words_lossless (WavpackStream *wps, int32_t *buffer, int32_t nsample
                 bs->bc -= (ones_count = ones_count_table [next8]) + 1;
                 bs->sr >>= ones_count + 1;
             }
+#else
+            for (ones_count = 0; ones_count < (LIMIT_ONES + 1) && getbit (bs); ++ones_count);
 
+            if (ones_count >= LIMIT_ONES) {
+                uint32_t mask;
+                int cbits;
+
+                if (ones_count == (LIMIT_ONES + 1))
+                    break;
+
+                for (cbits = 0; cbits < 33 && getbit (bs); ++cbits);
+
+                if (cbits == 33)
+                    break;
+
+                if (cbits < 2)
+                    ones_count = cbits;
+                else {
+                    for (mask = 1, ones_count = 0; --cbits; mask <<= 1)
+                        if (getbit (bs))
+                            ones_count |= mask;
+
+                    ones_count |= mask;
+                }
+
+                ones_count += LIMIT_ONES;
+            }
+#endif
             if (wps->w.holding_one) {
                 wps->w.holding_one = ones_count & 1;
                 ones_count = (ones_count >> 1) + 1;
